﻿EnableExplicit

;- Information.
; Dieses Modul ist das Hauptfenster des Projekts.

;- Version.
; Status:   In Arbeit.
; Autor:    Peter Dreisiebner
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-03-09 - PureBasic v5.60 (x86).


;-
;- DeclareModule WindowStart.
;-
DeclareModule WindowStart
   EnableExplicit

   Declare.i Open()

EndDeclareModule  ; WindowStart.

;
;- Include.
;

;-
;- Module WindowStart.
;-
Module WindowStart
   EnableExplicit
   UseModule Common
   UseModule Tool
   DebugOutWindowStart

   ;- Konstanten.
   #iWindowWidth = 800
   #iWindowHeight = 450
   #iWindowMinWidth = 150
   #iWindowMinHeight = 150
   CompilerIf #fDebugOut Or #fDebugSpeed
      #sWindowTitle = #sApplicationName + " (Debugversion)"
   CompilerElse
      #sWindowTitle = #sApplicationName
   CompilerEndIf

   #sFontName = "Courier New"
   #iFontSize = 9

   #sMenuTitleFile = "&Datei"
   #sMenuItemFileExit = "&Beenden"
   #sMenuTitleEdit = "&Bearbeiten"
   #sMenuTitleHelp = "&Hilfe"
   #sMenuItemHelpLog = "&Log..."
   #sMenuItemHelpAbout = "&Über..."
   Enumeration eMenuItemId
      #eMenuItemFileExit
      #eMenuItemHelpLog
      #eMenuItemHelpAbout
   EndEnumeration

   Enumeration
      #iListViewGetVersion
      #iListViewUsbTmcGetDeviceList
      #iListViewUsbTmcSendData
      #iListViewUsbTmcRequestData
   EndEnumeration

   ;- Variablen.
   Global miWindowNumber.i
   Global miMenuNumber.i
   Global miListViewGadgetNumber.i
   Global miEditorGadgetNumber.i
   Global miSplitterGadgetNumber.i

   ;- Declares.
   Declare LibUsbGetVersion()
   Declare Close()
   Declare UsbTmcGetDeviceList()
   Declare UsbTmcSendData()
   Declare UsbTmcRequestData()

   ;-

   Procedure.i EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMinimizeWindow()
      DebugView::ProcIn()

      WindowLog::Hide()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventSizeWindow()
      ; Ereignisprozedur wenn das Fenster in der Größe geändert wurde.
      ; Die Gadgets an die neue Größe des Fensters anpassen.
      DebugView::ProcIn()
      Protected iWidth.i, iHeight.i

      iWidth = WindowWidth(miWindowNumber)
      iHeight = (WindowHeight(miWindowNumber) - MenuHeight())

      If miSplitterGadgetNumber
         ResizeGadget(miSplitterGadgetNumber, #PB_Ignore, #PB_Ignore, iWidth - 20, iHeight - 20)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure EventRestoreWindow()
      DebugView::ProcIn()

      WindowLog::RestoreWindow()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuFileExit()
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpLog()
      DebugView::ProcIn()

      WindowLog::Show(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpAbout()
      DebugView::ProcIn()

      WindowAbout::Open(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure EventListViewGadgetLeftClick()
      ; Ereignis wenn ein Eintrag in dem ListViewGadget ausgewählt wurde.
      DebugView::ProcIn()
      Protected iItem.i

      iItem = GetGadgetState(miListViewGadgetNumber)
      Select iItem
         Case #iListViewGetVersion
            LibUsbGetVersion()
         Case #iListViewUsbTmcGetDeviceList
            UsbTmcGetDeviceList()
         Case #iListViewUsbTmcSendData
            UsbTmcSendData()
         Case #iListViewUsbTmcRequestData
            UsbTmcRequestData()
      EndSelect

      DebugView::ProcOut()
   EndProcedure

   ;-

   Procedure LibUsbGetVersion()
      DebugView::ProcIn()
      Protected fResult.i
      Protected sVersion.String, sDescription.String, sError.String

      ClearGadgetItems(miEditorGadgetNumber)

      If LibUsb::GetVersion(sVersion, sDescription, sError)
         AddGadgetItem(miEditorGadgetNumber, -1, "Version: " + sVersion\s)
         AddGadgetItem(miEditorGadgetNumber, -1, "Beschreibung: " + sDescription\s)
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure UsbTmcGetDeviceList()
      DebugView::ProcIn()
      Protected *uDevice.UsbTmc::uDevice, iUsbTmc.i, c.i, iDeviceCount, sError.String
      Protected uCapabilities.UsbTmc::uCapabilities
      NewList TmcDeviceList.UsbTmc::uDevice()

      ClearGadgetItems(miEditorGadgetNumber)

      iUsbTmc = UsbTmc::NewInstance(#Null, sError)
      If (Not iUsbTmc)
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)

      Else
         If UsbTmc::GetTmcDeviceList(iUsbTmc, TmcDeviceList(), @iDeviceCount, sError)
            If iDeviceCount
               ForEach TmcDeviceList()
                  c + 1
                  *uDevice = TmcDeviceList()
                  If *uDevice
                     AddGadgetItem(miEditorGadgetNumber, -1, "Gerät " + c)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  UsbTmc: " + StringBool(*uDevice\fUsbTmc))
                     AddGadgetItem(miEditorGadgetNumber, -1, "  VendorID: $" + Hex(*uDevice\iVendorId))
                     AddGadgetItem(miEditorGadgetNumber, -1, "  ProductID: $" + Hex(*uDevice\iProductId))
                     AddGadgetItem(miEditorGadgetNumber, -1, "  Manufacturer: " + *uDevice\sManufacturer)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  Product: " + *uDevice\sProduct)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  Serialnumber: " + *uDevice\sSerialnumber)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  BusNumber: " + *uDevice\iBusNumber)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  PortNumber: " + *uDevice\iPortNumber)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  InterfaceNumber: " + *uDevice\iInterfaceNumber)
                     AddGadgetItem(miEditorGadgetNumber, -1, "  EndpointInterruptIn: $" + Hex(*uDevice\iEndpointInterruptIn))
                     AddGadgetItem(miEditorGadgetNumber, -1, "  EndpointBulkIn: $" + Hex(*uDevice\iEndpointBulkIn))
                     AddGadgetItem(miEditorGadgetNumber, -1, "  EndpointBulkOut: $" + Hex(*uDevice\iEndpointBulkOut))
                     AddGadgetItem(miEditorGadgetNumber, -1, "")

                     If UsbTmc::SetTmcDevice(iUsbTmc, *uDevice, sError)
                        If (Not UsbTmc::GetCapabilities(iUsbTmc, @uCapabilities, sError))
                           AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                           WindowLog::Error(sError\s)

                        Else
                           AddGadgetItem(miEditorGadgetNumber, -1, "Gerät " + c + " - USBTMC-USB488-Capabilities")
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USBTMC BCD Version Number: $" + Hex(uCapabilities\iTmcBcdVersionNumber))
                           AddGadgetItem(miEditorGadgetNumber, -1, "   USBTMC Interface accepts the Indicator Pulse request: " + Tool::StringBool(uCapabilities\fTmcInterfaceAcceptsIndicatorPulseRequest))
                           AddGadgetItem(miEditorGadgetNumber, -1, "   USBTMC Interface is talk-only: " + StringBool(uCapabilities\fTmcInterfaceIsTalkOnly))
                           AddGadgetItem(miEditorGadgetNumber, -1, "   USBTMC Interface is listen-only: " + StringBool(uCapabilities\fTmcInterfaceIsListenOnly))
                           AddGadgetItem(miEditorGadgetNumber, -1, "   USBTMC Device supports ending TermChar: " + StringBool(uCapabilities\fTmcDeviceSupportsEndingTermChar))
                           AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 BCD Version Number: $" + Hex(uCapabilities\i488BcdVersionNumber))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Interface is a 488.2 USB488 Interface: " + StringBool(uCapabilities\f488InterfaceIs4882))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Interface accepts Remote/Local control: " + StringBool(uCapabilities\f488InterfaceAcceptsRemoteLocalControlRequests))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Interface accepts Trigger messages: " + StringBool(uCapabilities\f488InterfaceAcceptsTriggerMessage))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Device understands all mandatory SCPI commands: " + StringBool(uCapabilities\f488DeviceUnderstandsAllMandatoryScpiCommands))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Device is Service Request capable: " + StringBool(uCapabilities\f488DeviceIsServiceRequestCapable))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Device is Remote/Local capable: " + StringBool(uCapabilities\f488DeviceIsRemoteLocalCapable))
                           AddGadgetItem(miEditorGadgetNumber, -1, "  USB488 Device is Trigger capable: " + StringBool(uCapabilities\f488DeviceIsTriggerCapable))
                        EndIf
                     Else
                        AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                     EndIf
                  EndIf
               Next  ; ForEach.

            Else
               AddGadgetItem(miEditorGadgetNumber, -1, "Keine Geräte gefunden.")
            EndIf
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf

         If (Not UsbTmc::DeleteInstance(iUsbTmc, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure UsbTmcSendData()
      DebugView::ProcIn()
      Protected *uDevice.UsbTmc::uDevice, iUsbTmc.i, c.i, iDeviceCount, sError.String
      Protected uCapabilities.UsbTmc::uCapabilities
      NewList TmcDeviceList.UsbTmc::uDevice()

      ClearGadgetItems(miEditorGadgetNumber)

      iUsbTmc = UsbTmc::NewInstance(#Null, sError)
      If (Not iUsbTmc)
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
      Else
         If UsbTmc::GetTmcDeviceList(iUsbTmc, TmcDeviceList(), @iDeviceCount, sError)
            If iDeviceCount

               ForEach TmcDeviceList()
                  c + 1
                  *uDevice = TmcDeviceList()
                  If *uDevice
                     If ((*uDevice\iVendorId = $1AB1) And (*uDevice\iProductId = $3737))
                        AddGadgetItem(miEditorGadgetNumber, -1, "Gerät " + c)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  VendorID: $" + Hex(*uDevice\iVendorId))
                        AddGadgetItem(miEditorGadgetNumber, -1, "  ProductID: $" + Hex(*uDevice\iProductId))
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Manufacturer: " + *uDevice\sManufacturer)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Product: " + *uDevice\sProduct)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Serialnumber: " + *uDevice\sSerialnumber)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  BusNumber: " + *uDevice\iBusNumber)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  PortNumber: " + *uDevice\iPortNumber)

                        If UsbTmc::SetTmcDevice(iUsbTmc, *uDevice, sError)

                           Protected fResult.i, sScpi.String, *Data, iLength.i, iTransferred.i
                           sScpi\s = ":ROUTe:OPEN:ALL"
                           ;sScpi\s = "*IDN?"
                           ;*Data = Ascii("#+#+#") ; für binäre Testdaten.
                           ;iLength = MemoryStringLength(*Data, #PB_Ascii)
                           UsbTmc::SetTimeout(iUsbTmc, 500, sError)
                           UsbTmc::SetTerminationChar(iUsbTmc, $0A, #False, sError)

                           fResult = UsbTmc::PrepareData(iUsbTmc, sScpi, *Data, iLength, sError)
                           If (Not fResult)
                              AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                              WindowLog::Error(sError\s)
                           EndIf
                           If fResult
                              If (Not UsbTmc::SendData(iUsbTmc, UsbTmc::#eDevDepMsgOut, @iTransferred, sError))
                                 AddGadgetItem(miEditorGadgetNumber, -1, "Fehler bei UsbTmc::SendData():")
                                 AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                                 WindowLog::Error(sError\s)
                              Else
                                 AddGadgetItem(miEditorGadgetNumber, -1, "SCPI: " + sScpi\s)
                                 AddGadgetItem(miEditorGadgetNumber, -1, "Die Daten wurden erfolgreich gesendet. iTransferred = " + iTransferred)
                              EndIf
                           EndIf
                           If *Data
                              FreeMemory(*Data)
                           EndIf
                        Else
                           AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                        EndIf
                        Break ; ForEach.
                     EndIf
                  EndIf
               Next  ; ForEach.
               If (Not UsbTmc::DeleteInstance(iUsbTmc, sError))
                  AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
               EndIf
            Else
               AddGadgetItem(miEditorGadgetNumber, -1, "Keine Geräte gefunden.")
            EndIf
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
      EndIf ; (Not iUsbTmc)

      DebugView::ProcOut()
   EndProcedure


   Procedure UsbTmcRequestData()
      DebugView::ProcIn()
      Protected *uDevice.UsbTmc::uDevice, iUsbTmc.i, c.i, iDeviceCount, sError.String
      Protected uCapabilities.UsbTmc::uCapabilities
      NewList TmcDeviceList.UsbTmc::uDevice()
      Protected fResult.i, sScpi.String, *Data, iLength.i, iTransferred.i, *ReceivedData, sHex.s
      ClearGadgetItems(miEditorGadgetNumber)

      iUsbTmc = UsbTmc::NewInstance(#Null, sError)
      If (Not iUsbTmc)
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
      Else
         If UsbTmc::GetTmcDeviceList(iUsbTmc, TmcDeviceList(), @iDeviceCount, sError)
            If iDeviceCount

               ForEach TmcDeviceList()
                  c + 1
                  *uDevice = TmcDeviceList()
                  If *uDevice
                     ;If ((*uDevice\iVendorId = $1AB1) And (*uDevice\iProductId = $3737))
                     If (*uDevice\iVendorId = $1AB1)
                        AddGadgetItem(miEditorGadgetNumber, -1, "Gerät " + c)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  VendorID: $" + Hex(*uDevice\iVendorId))
                        AddGadgetItem(miEditorGadgetNumber, -1, "  ProductID: $" + Hex(*uDevice\iProductId))
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Manufacturer: " + *uDevice\sManufacturer)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Product: " + *uDevice\sProduct)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  Serialnumber: " + *uDevice\sSerialnumber)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  BusNumber: " + *uDevice\iBusNumber)
                        AddGadgetItem(miEditorGadgetNumber, -1, "  PortNumber: " + *uDevice\iPortNumber)

                        If UsbTmc::SetTmcDevice(iUsbTmc, *uDevice, sError)

                           sScpi\s = "*IDN?"
                           ;sScpi\s = ":WAV:SOUR CHAN1;:WAV:DATA?"
                           ;sScpi\s = ":DISP:DATA?"
                           ;*Data = Ascii("#+#+#") ; für binäre Testdaten.
                           ;iLength = MemoryStringLength(*Data, #PB_Ascii)
                           UsbTmc::SetTimeout(iUsbTmc, 500, sError)
                           UsbTmc::SetTerminationChar(iUsbTmc, $0A, #False, sError)

                           DebugView::SpeedStart()

                           fResult = UsbTmc::PrepareData(iUsbTmc, sScpi, *Data, iLength, sError)

                           DebugView::SpeedStop("UsbTmc::PrepareData")

                           If (Not fResult)
                              AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                              WindowLog::Error(sError\s)
                           EndIf
                           If fResult

                              DebugView::SpeedStart()

                              If (Not UsbTmc::SendData(iUsbTmc, UsbTmc::#eDevDepMsgOut, @iTransferred, sError))
                                 AddGadgetItem(miEditorGadgetNumber, -1, "Fehler bei UsbTmc::SendData():")
                                 AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                                 WindowLog::Error(sError\s)
                              Else
                                 DebugView::SpeedStop("UsbTmc::SendData")

                                 AddGadgetItem(miEditorGadgetNumber, -1, "SCPI: " + sScpi\s)
                                 AddGadgetItem(miEditorGadgetNumber, -1, "Die Daten wurden erfolgreich gesendet. iTransferred = " + iTransferred)
                              EndIf
                           EndIf
                           If *Data
                              FreeMemory(*Data)
                           EndIf

                           ;Delay(50)

                           DebugView::SpeedStart()

                           If (Not UsbTmc::RequestData(iUsbTmc, @*ReceivedData, @iTransferred, UsbTmc::#eDevDepMsgIn, sError))
                              AddGadgetItem(miEditorGadgetNumber, -1, "Fehler bei UsbTmc::RequestData():")
                              AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                              WindowLog::Error(sError\s)
                           Else
                              DebugView::SpeedStop("UsbTmc::RequestData")

                              AddGadgetItem(miEditorGadgetNumber, -1, "Die Daten wurden erfolgreich empfangen. iTransferred = " + iTransferred)

                              If *ReceivedData
                                 If (iTransferred > 256)
                                    sHex = Tool::MemoryFormatHex(*ReceivedData, 256)
                                    AddGadgetItem(miEditorGadgetNumber, -1, "Die ersten 256 Bytes:")
                                 Else
                                    sHex = Tool::MemoryFormatHex(*ReceivedData, iTransferred)
                                    AddGadgetItem(miEditorGadgetNumber, -1, "Die gesamten Daten:")
                                 EndIf
                                 AddGadgetItem(miEditorGadgetNumber, -1, sHex)
                              EndIf

                           EndIf

; ShowMemoryViewer(*ReceivedData, iTransferred)
; CallDebugger

                           If *ReceivedData
                              FreeMemory(*ReceivedData)
                           EndIf

                        Else
                           AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                        EndIf
                        Break ; ForEach.
                     EndIf
                  EndIf
               Next  ; ForEach.
               If (Not UsbTmc::DeleteInstance(iUsbTmc, sError))
                  AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
               EndIf
            Else
               AddGadgetItem(miEditorGadgetNumber, -1, "Keine Geräte gefunden.")
            EndIf
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
      EndIf ; (Not iUsbTmc)

      DebugView::ProcOut()
   EndProcedure

   ;-

   Procedure.i Open()
      ; Das Programmfenster laden und anzeigen.
      DebugView::ProcIn()
      Protected iFontNumber.i

      miWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowWidth, #iWindowHeight, #sWindowTitle,
                                  #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_SizeGadget)
      If miWindowNumber
         WindowBounds(miWindowNumber, #iWindowMinWidth, #iWindowMinHeight, #PB_Ignore, #PB_Ignore)

         BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         BindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         BindEvent(#PB_Event_SizeWindow, @EventSizeWindow(), miWindowNumber)
         BindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         Tool::WindowReduceFlicker(miWindowNumber)

         miMenuNumber = CreateMenu(#PB_Any, WindowID(miWindowNumber))
         If miMenuNumber
            MenuTitle(#sMenuTitleFile)
            MenuItem(#eMenuItemFileExit, #sMenuItemFileExit)
            MenuTitle(#sMenuTitleEdit)
            MenuTitle(#sMenuTitleHelp)
            MenuItem(#eMenuItemHelpLog, #sMenuItemHelpLog)
            MenuItem(#eMenuItemHelpAbout, #sMenuItemHelpAbout)

            BindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())
         EndIf

         iFontNumber = LoadFont(#PB_Any, #sFontName, #iFontSize)

         miListViewGadgetNumber = ListViewGadget(#PB_Any, 10, 10, 780, 210)
         If miListViewGadgetNumber
            SetGadgetFont(miListViewGadgetNumber, FontID(iFontNumber))

            AddGadgetItem(miListViewGadgetNumber, -1,
                          "LibUsb-Version - LibUsb::GetVersion()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewGetVersion)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "Alle verfügbaren USBTMC-Geräte auflisten - UsbTmc::GetTmcDeviceList() mit SetTmcDevice(), GetCapabilities()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewUsbTmcGetDeviceList)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "SCPI-Komando senden - UsbTmc::SendData() mit PrepareData()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewUsbTmcSendData)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "SCPI-Komando senden und empfangen - UsbTmc::SendData() und RequestData()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewUsbTmcRequestData)

            BindGadgetEvent(miListViewGadgetNumber, @EventListViewGadgetLeftClick(), #PB_EventType_LeftClick)
         EndIf

         miEditorGadgetNumber = EditorGadget(#PB_Any, 10, 230, 780, 210)
         SetGadgetFont(miEditorGadgetNumber, FontID(iFontNumber))

         miSplitterGadgetNumber = SplitterGadget(#PB_Any, 10, 10, 780, 430,
                                                 miListViewGadgetNumber, miEditorGadgetNumber) ;, #PB_Splitter_Separator)
         SetGadgetState(miSplitterGadgetNumber, 150)

         PostEvent(#PB_Event_SizeWindow, miWindowNumber, #Null)
      Else
         DebugView::Error("Das Fenster >" + #PB_Compiler_Module + "< konnte nicht erstellt werden.")
      EndIf

      DebugView::ProcOut()
      ProcedureReturn miWindowNumber
   EndProcedure


   Procedure Close()
      ; Schließt das Fenster und Programm.
      DebugView::ProcIn()

      If miWindowNumber
         UnbindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_SizeWindow, @EventSizeWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         UnbindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())

         UnbindGadgetEvent(miListViewGadgetNumber, @EventListViewGadgetLeftClick(), #PB_EventType_LeftClick)

         CloseWindow(miWindowNumber)
         miWindowNumber = #Null

         ; Ereignis für das Beenden des Programms an die Hauptschleife senden.
         PostEvent(#eCustomEventExitApplication)

      EndIf

      DebugView::ProcOut()
      ProcedureReturn
   EndProcedure


EndModule   ; WindowStart.

; IDE Options = PureBasic 5.60 (Windows - x86)
; Folding = ---
; EnableThread
; EnableXP
; EnableOnError
; CompileSourceDirectory
; EnablePurifier
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant
; EnableUnicode