﻿EnableExplicit

;- -- Information.
; Dieses Module sollte in jedes Projekt mit >XIncludeFile "..\..\_ PureBasic\WindowAbout-v01.pbi"<
; in die Hauptdatei, z.B. "Application.pb" eingebunden werden.
;
; Dieses Module ist das About/Über-Fenster des Projekts.
; Der angezeigte Text kann in der jeweiligen Common.pbi-Datei geändert werden.

;- -- Version.
; Status:   In Arbeit.
; Autor:    Peter Dreisiebner
; E-Mail:   web@dreisiebner.at
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-03-29 - PureBasic v5.60 (x86).


;-
;- -- DeclareModule WindowAbout. --------------------
;
DeclareModule WindowAbout
   EnableExplicit

   Declare.i Open(iParentWindowNumber.i)

EndDeclareModule  ; WindowAbout.

;-
;- -- Include.
;

;-
;- -- Module WindowAbout. --------------------
;
Module WindowAbout
   EnableExplicit
   UseModule Common
   DebugOutWindowAbout
   
   ;-
   ;- -- Konstanten.
   #iWindowWidth = 400
   #iWindowHeight = 400
   #sWindowTitle = "Über..."

   Enumeration eMenuItemId
      #eMenuItemShortcutEscape
      #eMenuItemShortcutEnter
   EndEnumeration
   
   ;-
   ;- -- Variablen.
   Global miWindowNumber.i
   Global miImageGadgetNumber.i
   Global miAboutImageNumber.i
   Global miEditorGadgetNumber.i
   Global miButtonGadgetNumber.i
   
   ;-
   ;- -- Declares.
   Declare Close()
   
   ;-

   Procedure.i EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuShortcut()
      ; Ereignisprozedur für das nicht vorhandene Menü.
      ; Es werden damit die Tastaturkürzel behandelt.
      DebugView::ProcIn()
      Protected iMenu.i

      iMenu = EventMenu()
      If (#eMenuItemShortcutEscape = iMenu) Or (#eMenuItemShortcutEnter = iMenu)
         Close()
      EndIf

      DebugView::ProcOut()
      ProcedureReturn
   EndProcedure


   Procedure EventButtonGadget()
      ; Ereignisprozedur für die OK-Schaltfläche.
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure.i Open(iParentWindowNumber.i)
      ; Das Fenster laden und anzeigen.
      DebugView::ProcIn()

      If (Not miWindowNumber)
         miWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowWidth, #iWindowHeight, #sWindowTitle,
                                     #PB_Window_SystemMenu | #PB_Window_WindowCentered, WindowID(iParentWindowNumber))
         If miWindowNumber
            Tool::WindowReduceFlicker(miWindowNumber)

            BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
            BindEvent(#PB_Event_Menu, @EventMenuShortcut(), miWindowNumber)
            AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Escape, #eMenuItemShortcutEscape)
            AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Return, #eMenuItemShortcutEnter)

            miAboutImageNumber = CatchImage(#PB_Any, ?WindowAboutImage, #iWindowAboutImageSize) ; siehe Common.pbi.
            If miAboutImageNumber
               miImageGadgetNumber = ImageGadget(#PB_Any, 10, 10, 380, 100, ImageID(miAboutImageNumber))
            Else
               DebugView::Error("Das Bild für das Über-Fenster konnte nicht von der 'DataSection' geladen werden.")
            EndIf

            miEditorGadgetNumber = EditorGadget(#PB_Any, 10, 120, 380, 235, #PB_Editor_ReadOnly | #ES_CENTER)
            If miEditorGadgetNumber
               Tool::EditorGadgetAllowOnlyPlaintext(miEditorGadgetNumber)
               ;SetGadgetText(miEditorGadgetNumber, #sWindowAboutText)  ; siehe Common.pbi.
               SetGadgetText(miEditorGadgetNumber,
                             ReplaceString(#sWindowAboutText, #sWindowAboutPureBasicVersionPlaceholder,
                                           StrF(#PB_Compiler_Version / 100, 2))) ; siehe Common.pbi.
            EndIf

            miButtonGadgetNumber = ButtonGadget(#PB_Any, 150, 365, 100, 25, "OK")
            If miButtonGadgetNumber
               BindGadgetEvent(miButtonGadgetNumber, @EventButtonGadget())
               SetActiveGadget(miButtonGadgetNumber)
            EndIf

         Else
            DebugView::Error("Das Fenster >" + #PB_Compiler_Module + "< konnte nicht erstellt werden.")
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn miWindowNumber
   EndProcedure


   Procedure Close()
      ; Schließt das Fenster.
      DebugView::ProcIn()

      UnbindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
      UnbindEvent(#PB_Event_Menu, @EventMenuShortcut(), miWindowNumber)

      UnbindGadgetEvent(miButtonGadgetNumber, @EventButtonGadget())

      RemoveKeyboardShortcut(miWindowNumber, #PB_Shortcut_All)

      FreeImage(miAboutImageNumber)

      CloseWindow(miWindowNumber)
      miWindowNumber = #Null

      DebugView::ProcOut()
   EndProcedure


EndModule   ; WindowAbout.
